C     ***********************************************************************************      
C     MAIN CODE FOR U.S. GEOLOGICAL SURVEY PARTICLE TRACKING FOR MODFLOW-LGR -- MPATH-LGR      
C     ***********************************************************************************      
C
C     THIS CODE USES MODPATH TO TRACK PARTICLES BETWEEN MODFLOW GRIDS
C     COUPLED USING THE LGR (LOCAL-GRID REFINEMENT) PACKAGE.
C     TRACKING FOR COUPLED GRIDS USING GHOST NODES COULD BE ADDED LATER.
C                                                                             
C        SPECIFICATIONS:                                                      
C     ------------------------------------------------------------------      
      
      IMPLICIT NONE

      INCLUDE 'openspec.inc'      
 
      
C-------ASSIGN VERSION NUMBER AND DATE
      CHARACTER*40 VERSION
      PARAMETER (VERSION='1.0 (7-2011)')

      CHARACTER*200 COMLIN,LINE
      CHARACTER*80 HEADERLINE
      CHARACTER*80 VER(1)
C*************************************************************************************
     
C     ASSIGN FILE NAMES

C-------FILE NAMES FOR MODPATH-LGR
      CHARACTER*80 FNRSP                   ! MPLGR response filename
      CHARACTER*80 FNLGR                   ! LGR control filename
      CHARACTER*80 MPLGRSUMMARY
 
C-------FOR PARENT GRID
      CHARACTER*80 FNNAMPARENT         ! parent name file
      CHARACTER*80 FNDISPARENT         ! parent DIS file
      CHARACTER*80 FNRSPPARENT         ! parent response filename
      CHARACTER*80 FNENDPARENT         ! parent endpoint filename
      CHARACTER*80 FNMPNAMPARENT       ! parent MODPATH name filename
      CHARACTER*80 FNMAINPARENT        ! parent MODPATH MAIN filename 
      CHARACTER*80 FNBUDPARENTOLD      ! parent unmodified budget file
      CHARACTER*80 FNBUDPARENTNEW      ! parent modified budget file for shared nodes
      CHARACTER*80 FNPATHPARENT        ! parent pathline file
      CHARACTER*80 FNPATHPARENTOLD     ! parent scratch pathile file
      CHARACTER*80 FNTSPARENT          ! parent time-series file
      CHARACTER*80 FNTSPARENTOLD       ! parent scratch time-series file
      CHARACTER*80 FNPROJPARENT        ! parent projections file
      CHARACTER*80 FNPROJPARENTOLD     ! parent scratch projections file
      CHARACTER*80 FNENDPARENTOLD      ! parent scratch endpoint file
      CHARACTER*80 FNSTARTPARENT       ! parent starting locations file
      CHARACTER*80 FNSTARTPARENTOLD    ! parent scratch starting-locations file  
      CHARACTER*80 FNSTARTPARENTINITIAL ! parent first starting-locations file 

C-------FOR CHILD GRIDS
      CHARACTER*80 FNNAMCHILD         ! child name file     
      CHARACTER*80 FNDISCHILD         ! child DIS file
      CHARACTER*80 FNRSPCHILD         ! child response file 
      CHARACTER*80 FNENDCHILD         ! child endpoint file 
      CHARACTER*80 FNMPNAMCHILD       ! child MODPATH name filename
      CHARACTER*80 FNMAINCHILD        ! child MODPATH MAIN filename
      CHARACTER*80 FNBUDCHILDOLD      ! child unused file
      CHARACTER*80 FNBUDCHILDNEW      ! child unused file
      CHARACTER*80 FNPATHCHILD        ! child pathline file
      CHARACTER*80 FNPATHCHILDOLD     ! child scratch pathile file
      CHARACTER*80 FNTSCHILD          ! child time-series file
      CHARACTER*80 FNTSCHILDOLD       ! child scratch time-series file
      CHARACTER*80 FNPROJCHILD        ! child projections file
      CHARACTER*80 FNPROJCHILDOLD     ! child scratch projections file
      CHARACTER*80 FNENDCHILDOLD      ! child scratch endpoint file
      CHARACTER*80 FNSTARTCHILD       ! child starting locations file
      CHARACTER*80 FNSTARTCHILDOLD    ! child scratch starting locations file
      CHARACTER*80 FNSTARTCHILDINITIAL ! child first starting-locations file
      
      character*40 dummy,TEXT1,TEXT2,TEXT3
    
C-------GLOBAL OUTPUT FILES
      CHARACTER*80 FNGLOBEP
      CHARACTER*80 FNGLOBTS
      CHARACTER*80 FNGLOBPL
      CHARACTER*80 FNGLOBPROJ
      CHARACTER*75 PLHEADER
      CHARACTER*75 EPHEADER      
      CHARACTER*75 TSHEADER 
            
      INTEGER INNN

C-------SHARED NODE/GHOST NODE VARIABLES
      INTEGER NBDYTYP
      LOGICAL ISSHDNOD

C-------DIS PACKAGE VARIABLES          
      INTEGER BDYFLG,IUNITLGR,NIUNIT,IUNIT,NIUNITLGR
      INTEGER ILGR,IPART,IPCODE,IZONE,IZONE2
      INTEGER KFRST,IFRST,JFRST,I,J,K
      INTEGER IUDIS,INUNIT
      INTEGER NCOL,NROW,NLAY,NPER,NSTP
      INTEGER LBOTM,LAYCBD,NBOTM

C1------FOR PARENT GRID
      INTEGER NCOLP,NROWP,NLAYP,NPERP,NSTPP,ISSFLGP,LBOTMP,NBOTMP
      INTEGER IUDISPARENT
      REAL TOPP,BOTMP,DELRP,DELCP,PERLENP,TSMULTP

C2------FOR CHILD GRIDS
      INTEGER NROWC,NCOLC,NLAYC,NPERC,NSTPC,ISSFLGC
      INTEGER IUDISCHILD, NCUNIT
      REAL TOPC,BOTMC,DELRC,DELCC,PERLENC,TSMULTC

C-------MODFLOW-LGR VARIABLES

      INTEGER IBFLG              ! UNIT NUMBER FOR DEFINING SHARED MODFLOW-LGR CELLS
      INTEGER ISLGR              ! =0 RUN MODPATH FOR ONE GRID; >0 LGR GRIDS
      INTEGER NGRIDS             ! NUMBER OF GRIDS
      LOGICAL ILGRSTAT           ! LGR or non-LGR variables
      INTEGER NPCBEG,NPREND,NCPP ! VARIABLES FROM MODFLOW-LGR
      INTEGER IEXIT              ! =1 IF EXITING MODPATH-LGR SEQUENCES

C-------MODPATH-LGR VARIABLES
      CHARACTER NLG*4               ! COUNTER FOR CHILD GRIDS
      CHARACTER FMT*20              ! FORMAT STATUS FOR WRITING GLOBAL OUTPUTS
      INTEGER INTFLG                ! FLAG FOR INTERACTIVE INPUT
      INTEGER LG,LLG                ! COUNTER FOR CHILD GRIDS
      INTEGER NUMPARENTPART         ! NUMBER OF PARENT PARTICLES
      INTEGER NUMCHILDPART          ! NUMBER OF CHILD PARTICLES
      INTEGER NUMCHILDPARTTOTAL     ! TOTAL NUMBER OF CHILD PARTICLES
      INTEGER NUMPARENTPARTTOTAL    ! TOTAL NUMBER OF PARENT PARTICLES
      INTEGER IDMODEL_START,IDMODEL ! MODEL ID NUMBER

C-----LAY,ROW,COL AND X,Y,Z INFO FOR PARENT GRID

      REAL PXORIG,PYORIG,PZORIG
      REAL CXORIG,CYORIG,CZORIG
      REAL CXOFF,CYOFF
      REAL PXOFF,PYOFF
      REAL PXMAX,PYMAX
      REAL CXMAX,CYMAX

C-------VALUES FOR MODPATH RUNS FOR GRIDS
      INTEGER NUMITER,ITER,MAXSEQ,NMPSTOP,MAXPTS
      CHARACTER*4 NITER
      
C-------VALUES FOR COUNTERS FOR STRESS PERIODS, TIME STEPS
      INTEGER NPERRUN,NSTPRUN,NPTPPER
      CHARACTER*4 NSTPCOUNT,NPERCOUNT

C-------VALUES FOR STORING COUNTERS AND TO TALLY RESULTS      
      INTEGER PPARTNUM, CPARTNUM
     
C-------VALUES FOR STORING LGR INFO
      ALLOCATABLE NPCBEG(:)
      ALLOCATABLE NPREND(:)
      ALLOCATABLE NCPP(:)
      ALLOCATABLE IBFLG(:)

C1------FOR PARENT GRIDS      
      ALLOCATABLE TOPP(:,:)
      ALLOCATABLE BOTMP(:,:,:)
      ALLOCATABLE DELRP(:)
      ALLOCATABLE DELCP(:)
      ALLOCATABLE PERLENP(:)
      ALLOCATABLE NSTPP(:)
      ALLOCATABLE TSMULTP(:) 
      ALLOCATABLE ISSFLGP(:)
      ALLOCATABLE PXMAX(:)
      ALLOCATABLE PYMAX(:)
      
C2------FOR CHILD GRIDS
      ALLOCATABLE NROWC(:)
      ALLOCATABLE NCOLC(:)
      ALLOCATABLE NLAYC(:)
      ALLOCATABLE NPERC(:)
      ALLOCATABLE NSTPC(:,:)
      ALLOCATABLE ISSFLGC(:,:)     
                         
      ALLOCATABLE TOPC(:,:,:)
      ALLOCATABLE BOTMC(:,:,:,:)
      ALLOCATABLE DELRC(:,:)
      ALLOCATABLE DELCC(:,:)
      ALLOCATABLE PERLENC(:,:)
      ALLOCATABLE TSMULTC(:,:) 
      ALLOCATABLE IUDISCHILD(:)
      ALLOCATABLE CXMAX(:,:)
      ALLOCATABLE CYMAX(:,:)

C-------VALUES FOR MODPATH-LGR      
      ALLOCATABLE FNNAMCHILD(:)         ! child name file     
      ALLOCATABLE FNDISCHILD(:)         ! child DIS file
      ALLOCATABLE FNRSPCHILD(:)         ! child response file 
      ALLOCATABLE FNENDCHILD(:)         ! child endpoint file 
      ALLOCATABLE FNMPNAMCHILD(:)       ! child MODPATH name filename
      ALLOCATABLE FNMAINCHILD(:)        ! child MODPATH MAIN filename
      ALLOCATABLE FNBUDCHILDOLD(:)      ! child unused file
      ALLOCATABLE FNBUDCHILDNEW(:)      ! child unused file
      ALLOCATABLE FNPATHCHILD(:)        ! child pathline file
      ALLOCATABLE FNPATHCHILDOLD(:)     ! child scratch pathile file
      ALLOCATABLE FNTSCHILD(:)          ! child time-series file
      ALLOCATABLE FNTSCHILDOLD(:)       ! child scratch time-series file
      ALLOCATABLE FNPROJCHILD(:)        ! child projection file
      ALLOCATABLE FNPROJCHILDOLD(:)     ! child scratch projections file
      ALLOCATABLE FNENDCHILDOLD(:)      ! child scratch endpoint file
      ALLOCATABLE FNSTARTCHILD(:)       ! child starting locations file
      ALLOCATABLE FNSTARTCHILDOLD(:)    ! child scratch starting locations file
      ALLOCATABLE FNSTARTCHILDINITIAL(:) ! child first starting-locations file

      ALLOCATABLE CXOFF(:)
      ALLOCATABLE CYOFF(:)
      ALLOCATABLE PXOFF(:)
      ALLOCATABLE PYOFF(:)
      
      ALLOCATABLE NUMCHILDPART(:)
      ALLOCATABLE NUMPARENTPART(:)
      ALLOCATABLE NUMCHILDPARTTOTAL(:)
      ALLOCATABLE NUMPARENTPARTTOTAL(:)
      ALLOCATABLE PPARTNUM(:,:)
      ALLOCATABLE CPARTNUM(:,:)

      PARAMETER (NIUNIT=100)
      PARAMETER (NIUNITLGR=100)
      PARAMETER (MAXSEQ=10)

      DIMENSION IUNITLGR(NIUNITLGR)      
      DIMENSION IUNIT(NIUNIT)

C-----VARIABLES FOR READING FILE NAMES
      INTEGER ICOL,N,ISTART,ISTOP,LLOC,ITYP1,ITYP2,IU,IDUM,
     1 INAM1,INAM2,IFLEN,IOUT, IOUTG,NC,INDIS,IN,ISS,ICMPCTP,ICMPCTC,
     2 IOUTEPGL,IOUTTSGL,IOUTPLGL
C
      REAL        R, TREF, TIMX
C
      LOGICAL EXISTS, FILE_EXISTS


      CHARACTER(LEN=4) :: CUNIT(NIUNIT),CDUM
      DATA CUNIT/'BCF6', 'WEL ', 'DRN ', 'RIV ', 'EVT ', '    ', 'GHB ',  !  7
     &           'RCH ', 'SIP ', 'DE4 ', 'SOR ', 'OC  ', 'PCG ', 'LMG ',  ! 14
     &           'GWT ', 'FHB ', 'RES ', 'STR ', 'IBS ', 'CHD ', 'HFB6',  ! 21
     &           'LAK ', 'LPF ', 'DIS ', 'SEN ', 'PES ', 'OBS ', 'HOB ',  ! 28
     &           'ADV2', 'COB ', 'ZONE', 'MULT', 'DROB', 'RVOB', 'GBOB',  ! 35
     &           'STOB', 'HUF ', 'CHOB', 'ETS ', 'DRT ', 'DTOB', '    ',  ! 42
     &           'HYD ', '    ', 'SFOB', 'GAGE', '    ', '    ', 'LMT6',  ! 49
     &           '    ', 50*'    '/                                       ! 56
C     ------------------------------------------------------------------

C     ASSIGN FILE NAME FOR INPUT ECHO
      MPLGRSUMMARY='MPLGR_SUMMARY.TXT'

C     RESERVE AND SET UNIT NUMBERS
C-----UNIT NUMBER FOR DIS FILE FROM NAME FILE
      IUDIS = 24
      
C-----READ INPUT FROM THE MODPATH-LGR CONTROL FILE
      IUNITLGR(1)=75

C-----READ INPUT FROM A MF-LGR CONTROL FILE
      IUNITLGR(2)=76
      
C-----READ INPUT FROM A MODPATH RESPONSE FILE
      IUNITLGR(3)=77

C-----READ INPUT FROM A MODPATH NAME FILE IUNITLGR(4)
      IUNITLGR(4)=78

C-----READ INPUT FROM AN OLD BUDGET FILE
      IUNITLGR(5)=78

C-----READ INPUT FROM AN NEW BUDGET FILE
      IUNITLGR(6)=79

C-----READ INPUT FROM A DIS FILE
      IUNITLGR(7)= 78

C-----READ INPUT FROM AN OLD ENDPOINT FILE
      IUNITLGR(8)=79

C-----FORMATTED SCRATCH FILE
      IUNITLGR(9)=75  

C-----UNFORMATTED STARTING LOCATIONS FOR PARENT     
      IUNITLGR(10)=76
 
C-----UNFORMATTED STARTING LOCATIONS FOR CHILD1     
      IUNITLGR(11)=77

C-----UNFORMATTED STARTING LOCATIONS FOR CHILD2     
      IUNITLGR(12)=78
      
C-----UNFORMATTED STARTING LOCATIONS FOR CHILD3     
      IUNITLGR(13)=79

C-----UNFORMATTED STARTING LOCATIONS FOR CHILD4     
      IUNITLGR(14)=80

C-----UNFORMATTED STARTING LOCATIONS FOR CHILD5     
      IUNITLGR(15)=81

C-----ECHO INPUTS TO SUMMARY FILE MPLGRSUMMARY
      IUNITLGR(16)=82
      
C-----ECHO INPUTS TO SUMMARY FILE IOUT
      IOUT=IUNITLGR(16)

C ----WRITE UNFORMATTED STORAGE OF ENDPOINTS
      IUNITLGR(17)=83

C ----WRITE GLOBAL ENDPOINT FILE ON IUNITLGR(18)

C ----WRITE GLOBAL TIMESERIES FILE ON IUNITLGR(19)
      
C ----WRITE GLOBAL PATHLINE FILE ON IUNITLGR(20)
      
C ----WRITE GLOBAL PROJECTION FILE ON IUNITLGR(21)

C     OPEN FILE FOR INPUT ECHO
      OPEN(UNIT=IUNITLGR(16),FILE=MPLGRSUMMARY)
      IOUT=IUNITLGR(16)
      WRITE(*,*) ''
      WRITE(*,20) VERSION
      WRITE(IOUT,20) VERSION
      WRITE(IOUT,*) ''
   20 FORMAT('MODPATH-LGR VERSION ', A)           
C                                                                            

C************************************************************************

C     INTFLG = 1 NO INTERACTIVE INPUT, INITIALLY SET TO 1                         
C     INTFLG = 0 INTERACTIVE INPUT AND RUN FOR ONLY ONE MODEL
      INTFLG = 1

C     ISLGR = 0 ONE GRID, INITIALLY SET TO 0
C     ISLGR = 1 MULTIPLE GRIDS     
      ISLGR = 0 

C     Get MODPATH-LGR response file name from command line or user interaction.      
      CALL GETLGRRSP(FNRSP,INTFLG)   

C     IF INTERACTIVE INPUT IS NOT SPECIFIED AT COMMAND LINE, DETERMINE IF 
C     RUNNING MODPATH-LGR FOR ONE GRID OR LGR GRIDS                                                          
C-------OPEN MPLGR RESPONSE FILE                                                                
      IF(INTFLG.EQ.1) THEN
         CALL ISSLGR(IUNITLGR(1),IUNITLGR(2),IUNITLGR(16),FNRSP,FNLGR,
     1    ISLGR,NGRIDS)
      END IF
C-----If no LGR, run MODPATH for one grid.
      IF(ISLGR.EQ.0) THEN
          CALL MPATH(ISLGR,FNRSP,INTFLG,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     1     0,0,0,1,0,1,0)
          STOP        
       END IF
     
C-------ALLOCATE CHILD GRID ARRAYS FOR MODFLOW-LGR INFO
      ALLOCATE (NPCBEG(NGRIDS-1))
      ALLOCATE (NPREND(NGRIDS-1))
      ALLOCATE (NCPP(NGRIDS-1))
      ALLOCATE (IBFLG(NGRIDS-1))
      ALLOCATE (IUDISCHILD(NGRIDS-1))
      
C-------ALLOCATE CHILD GRID ARRAYS FOR MODPATH-LGR      
      ALLOCATE (FNNAMCHILD(NGRIDS-1))         ! child name file
      ALLOCATE (FNDISCHILD(NGRIDS-1))         ! child DIS file     
      ALLOCATE (FNRSPCHILD(NGRIDS-1))         ! child response file 
      ALLOCATE (FNENDCHILD(NGRIDS-1))         ! child endpoint file 
      ALLOCATE (FNMPNAMCHILD(NGRIDS-1))       ! child MODPATH name filename
      ALLOCATE (FNMAINCHILD(NGRIDS-1))        ! child MODPATH MAIN filename
      ALLOCATE (FNBUDCHILDOLD(NGRIDS-1))      ! child unused file
      ALLOCATE (FNBUDCHILDNEW(NGRIDS-1))      ! child unused file
      ALLOCATE (FNPATHCHILD(NGRIDS-1))        ! child pathline file
      ALLOCATE (FNPATHCHILDOLD(NGRIDS-1))     ! child scratch pathile file
      ALLOCATE (FNTSCHILD(NGRIDS-1))          ! child time-series file
      ALLOCATE (FNTSCHILDOLD(NGRIDS-1))       ! child scratch time-series file
      ALLOCATE (FNPROJCHILD(NGRIDS-1))        ! child projection file
      ALLOCATE (FNPROJCHILDOLD(NGRIDS-1))     ! child scratch projection file
      ALLOCATE (FNENDCHILDOLD(NGRIDS-1))      ! child scratch endpoint file
      ALLOCATE (FNSTARTCHILD(NGRIDS-1))       ! child starting locations file
      ALLOCATE (FNSTARTCHILDOLD(NGRIDS-1))    ! child scratch starting locations file
      ALLOCATE (FNSTARTCHILDINITIAL(NGRIDS-1)) ! child first starting-locations file
      ALLOCATE (NUMPARENTPART(NGRIDS-1))       ! NUMBER OF PARENT PARTICLES FROM CHILD MODELS
      ALLOCATE (NUMCHILDPART(NGRIDS-1))       ! NUMBER OF CHILD PARTICLES FROM PARENT MODEL

      ALLOCATE (NROWC(NGRIDS-1))
      ALLOCATE (NCOLC(NGRIDS-1))
      ALLOCATE (NLAYC(NGRIDS-1))
      ALLOCATE (NPERC(NGRIDS-1))
      
      ALLOCATE (CXOFF(NGRIDS-1))
      ALLOCATE (CYOFF(NGRIDS-1))
      ALLOCATE (PXOFF(NGRIDS-1))
      ALLOCATE (PYOFF(NGRIDS-1))
      
      ALLOCATE (PPARTNUM(MAXSEQ,NGRIDS-1))
      ALLOCATE (CPARTNUM(MAXSEQ,NGRIDS-1))
      ALLOCATE (NUMPARENTPARTTOTAL(MAXSEQ))      
      ALLOCATE (NUMCHILDPARTTOTAL(MAXSEQ))
      
           
      CALL GETLGR(IUNITLGR(2),FNLGR,FNNAMPARENT,FNNAMCHILD,NGRIDS,
     1 NPCBEG,NPREND,NCPP) ! read MFLGR control file
      CLOSE(IUNITLGR(2))
      
C     SET LGR METHOD TO SHARED NODE FOR NOW...
C     READ A FLAG TO INDICATE IF LGR USES SHARED NODES OR GHOST NODES
C      ISSHDNOD=.FALSE.
C      READ(IUNITLGR(1),*) NBDYTYP
C-----LGRFLG=0, LGR USES SHARED NODE METHOD
C-----LGRFLG=1, LGR USES GHOST NODE METHOD
C      IF(NBDYTYP.NE.0.AND.NBDYTYP.NE.1) THEN
C      WRITE (*,150) 
C  150 FORMAT('VALUE OF NBDYTYP MUST BE EITHER 0 or 1') 
C      STOP
C      END IF
C      IF(NBDYTYP.EQ.0) THEN
         ISSHDNOD=.TRUE.
C         WRITE(IUNITLGR(16),160)
C      ELSE
C         WRITE(IUNITLGR(16),170)
C  160 FORMAT('USING THE SHARED-NODE METHOD')
C  170 FORMAT('USING THE GHOST-NODE METHOD')   
C      END IF

C-----READ MPLGR CONTROL FILE FOR RESPONSE FILE NAME FOR THE PARENT GRID                     
      CALL GETFNAM(IUNITLGR(1),FNRSPPARENT,'.RSP')
      WRITE(IOUT,210) TRIM(ADJUSTL(FNRSPPARENT))
  210 FORMAT('MODPATH RESPONSE FILE FOR THE PARENT GRID: <',A,'>') 

C     WHEN USING THE SHARED NODE METHOD, THE PARENT BOUNDARY FLUXES 
C     ARE INCREASED TO ACCOUNT FOR HALVED CONDUCTANCES AT THE LGR BOUNDARY.
C     THE FLUXES ARE INCREASED BY SUBROUTINE PCBC.  USER MUST PROVIDE THE
C     UNMODIFIED PARENT BUDGET FILE TO THE MPLGR CONTROL FILE

C----------READ MODPATH NAME FILE FOR PARENT FILES
         OPEN(IUNITLGR(3),FILE=FNRSPPARENT,STATUS='OLD')
         READ(IUNITLGR(3),'(A)') TEXT1
         READ(IUNITLGR(3),'(A)') TEXT1
         READ(IUNITLGR(3),'(A)') TEXT1
         CALL GETFNAM(IUNITLGR(3),FNMPNAMPARENT,'.NAM')
         CLOSE(IUNITLGR(3))

         OPEN(IUNITLGR(3),FILE=FNMPNAMPARENT,STATUS='OLD')
         CALL GETMP(IUNITLGR(3),FNMAINPARENT,FNBUDPARENTNEW,
     1    FNSTARTPARENT,FNPATHPARENT,FNENDPARENT,FNTSPARENT,
     2    FNPROJPARENT,IU,IU,IU,IU)
         CLOSE(IUNITLGR(3))
         
    !!! ADD READ OF MAIN DATA FILE TO GET OUTPUT FORMAT     

          IF(ISSHDNOD) THEN
              WRITE(IUNITLGR(16),320) TRIM(ADJUSTL(FNBUDPARENTNEW)) 
  320         FORMAT('MODIFIED PARENT BUDGET FILE:' ,15X, '<',A,'>')
          END IF
      IF(ISSHDNOD) THEN
C--------READ MPLGR CONTROL FILE FOR PARENT UNMODIFIED BUDGET FILE 
         CALL GETFNAM(IUNITLGR(1),FNBUDPARENTOLD,'.BUD')
         WRITE(IOUT,300) TRIM(ADJUSTL(FNBUDPARENTOLD))
         WRITE(IOUT,*)          
  300    FORMAT('UNMODIFIED PARENT BUDGET FILE:', 13X, '<',A,'>')
      END IF

C********************************************************************  

C     REWRITE BUDGET FILES FOR DOUBLED FLUX AT INTERFACE
C-----FOR PARENT GRID IF USING THE SHARED-NODE METHOD FOR LGR.
C     THE BOUNDARY CORRECTION IS NOT NEEDED FOR CHILD GRIDS

      IF (ISSHDNOD) THEN
          CALL PCBC(FNBUDPARENTOLD,FNBUDPARENTNEW,FNLGR,IUNITLGR(5),
     1     IUNITLGR(6),IUNITLGR(2))
      END IF

C------READ MODPATH-LGR CONTROL FILE FOR CHILD GRID RESPONSE FILE NAMES 
      DO LG=1,NGRIDS-1

         CALL GETFNAM(IUNITLGR(1),FNRSPCHILD(LG),'.RSP')
C----------READ MODPATH NAME FILE FOR CHILD FILES

         OPEN(IUNITLGR(3),FILE=FNRSPCHILD(LG),STATUS='OLD')
         READ(IUNITLGR(3),'(A)') TEXT1
         READ(IUNITLGR(3),'(A)') TEXT1
         READ(IUNITLGR(3),'(A)') TEXT1
         CALL GETFNAM(IUNITLGR(3),FNMPNAMCHILD(LG),'.NAM')
         CLOSE(IUNITLGR(3))
         
         OPEN(IUNITLGR(3),FILE=FNMPNAMCHILD(LG),STATUS='OLD')
         CALL GETMP(IUNITLGR(3),FNMAINCHILD(LG),FNBUDCHILDNEW(LG),
     1    FNSTARTCHILD(LG),FNPATHCHILD(LG),FNENDCHILD(LG),FNTSCHILD(LG),
     2    FNPROJCHILD(LG),IU,IU,IU,IU)

         CLOSE(IUNITLGR(3))
   
C----------READ INTEGER VALUE IBFLG FOR BOUNDARY CELLS INDICATED IN IBOUND ARRAY
C----------FOR CHILD GRID ICHILD
  340     READ(IUNITLGR(1),'(A)') LINE                                  
          IF(LINE(1:1).EQ.'#') GOTO 340                            
          ICOL=1                                                  
          CALL URWORD(LINE,ICOL,ISTART,ISTOP,2,IBFLG(LG),R,0,
     1     IUNITLGR(1))

  350 FORMAT('MODPATH RESPONSE FILE FOR THE CHILD GRID:  <',A,'>') 
  355 FORMAT('MODPATH RESPONSE FILE FOR CHILD GRID'
     1 ,1X,I1,':'4X,'<',A,'>')
  360 FORMAT('IBFLG FOR THE CHILD GRID:',14X,I5)
  365 FORMAT('IBFLG FOR CHILD GRID',1X,I1,':',17X,I5) 

          IF(NGRIDS.EQ.2) THEN
              WRITE(IOUT,350) TRIM(ADJUSTL(FNRSPCHILD(LG)))
              WRITE(IOUT,360) IBFLG(LG) 
              WRITE(IOUT,*) ''
          ELSE
              WRITE(IOUT,355) LG,TRIM(ADJUSTL(FNRSPCHILD(LG)))
              WRITE(IOUT,365) LG,IBFLG(LG)
              WRITE(IOUT,*) ''
          END IF
      END DO ! END READING CHILD GRID INFO

C*********************************************************************      
C-------READ DIS FILE NAMES

C-------FOR PARENT GRID
C     OPEN THE NAME FILE FOR THE PARENT GRID      
      OPEN (UNIT=IUNITLGR(4),FILE=FNNAMPARENT,STATUS='OLD')
C     PARSE THE NAME FILE AND OPEN AND READ DISCRETIZATION PACKAGE. 
      CALL GETDIS(IUNITLGR(4),IUNIT,CUNIT,NIUNIT,IUDIS,FNDISPARENT,
     1 IUDISPARENT,IUNITLGR(16))
      
      WRITE(IOUT,370) IUDISPARENT
  370 FORMAT(1X,/1X,'PARENT GRID DISCRETIZATION INPUT DATA READ FROM',
     1 ' UNIT',I3)
     
      CLOSE(IUNITLGR(4))
      
C-------GET PARENT DIS INFORMATION
C     OPEN THE PARENT DIS FILE
      OPEN(UNIT=IUDISPARENT,FILE=FNDISPARENT,STATUS='OLD')
     
C-----Allocate space for discretization arrays                  
C1A    Note that NBOTM+1 arrays are allocated for BOTM           
C1B    because BOTM(I,J,0) contains the top elevation of layer 1.
      CALL ARDIS(IUDISPARENT,FNDISPARENT,NCOLP,NROWP,NLAYP,NPERP,IOUT)
      ALLOCATE (DELRP(NCOLP))
      ALLOCATE (DELCP(NROWP))
      ALLOCATE (PXMAX(NCOLP+1))
      ALLOCATE (PYMAX(NROWP+1))      
      ALLOCATE (BOTMP(NROWP,NCOLP,NLAYP+1))
      ALLOCATE (TOPP(NROWP,NCOLP))
      ALLOCATE (PERLENP(NPERP))
      ALLOCATE (NSTPP(NPERP))
      ALLOCATE (TSMULTP(NPERP))
      ALLOCATE (ISSFLGP(NPERP))

C-------READ GRID INFORMATION FOR THE PARENT GRID
      CALL RDDIS(IUDISPARENT,NCOLP,NROWP,NLAYP,NPERP,DELRP,DELCP,
     1 TOPP,BOTMP,PERLENP,NSTPP,TSMULTP,ISSFLGP,IUNITLGR(16))
  
      CLOSE (IUDISPARENT)

C---------CALCULATE MAX X and Y COORDINATES FOR EACH PARENT CELL
      CALL GRIDCOORD(NCOLP,NROWP,DELRP,DELCP,
     1 PXMAX,PYMAX)
C--------READ PARENT MAIN DATA FILE TO READ OPTIONS 
      CALL GETMAIN(IUNITLGR(7),FNMAINPARENT,ICMPCTP)      

C----------------------------------------------------------------------
C-------FOR CHILD GRID(S)
C     OPEN THE NAME FILE FOR THE CHILD GRIDS      
      DO LG=1,NGRIDS-1
          OPEN (UNIT=IUNITLGR(4),FILE=FNNAMCHILD(LG),STATUS='OLD')
C         PARSE THE NAME FILE AND OPEN AND READ DISCRETIZATION PACKAGE. 
          CALL GETDIS(IUNITLGR(4),IUNIT,CUNIT,NIUNIT,IUDIS,
     1     FNDISCHILD(LG),IUDISCHILD(LG),IUNITLGR(16))
          CLOSE(IUNITLGR(4))
      END DO

C-------GET CHILD DIS INFORMATION      

      DO LG=1,NGRIDS-1
C         OPEN THE CHILD DIS FILE
          OPEN(UNIT=IUDISCHILD(LG),FILE=FNDISCHILD(LG),STATUS='OLD')
          CALL ARDIS(IUDISCHILD(LG),FNDISCHILD(LG),NCOLC(LG),NROWC(LG),
     1     NLAYC(LG),NPERC(LG),IOUT)
      END DO
     
C---------Allocate space for discretization arrays                  
C1A       Note that NBOTM+1 arrays are allocated for BOTM           
C1B       because BOTM(I,J,0) contains the top elevation of layer 1.

      ALLOCATE (DELCC(NGRIDS-1,NROWC(NGRIDS-1)))
      ALLOCATE (DELRC(NGRIDS-1,NCOLC(NGRIDS-1)))
      ALLOCATE (CXMAX(NGRIDS-1,NCOLC(NGRIDS-1)+1))
      ALLOCATE (CYMAX(NGRIDS-1,NROWC(NGRIDS-1)+1))
      ALLOCATE (BOTMC(NGRIDS-1,NROWC(NGRIDS-1),NCOLC(NGRIDS-1),
     1 NLAYC(NGRIDS-1)+1))
      ALLOCATE (TOPC(NGRIDS-1,NROWC(NGRIDS-1),NCOLC(NGRIDS-1)))
      ALLOCATE (PERLENC(NGRIDS-1,NPERC(NGRIDS-1)))
      ALLOCATE (NSTPC(NGRIDS-1,NPERC(NGRIDS-1)))
      ALLOCATE (TSMULTC(NGRIDS-1,NPERC(NGRIDS-1)))
      ALLOCATE (ISSFLGC(NGRIDS-1,NPERC(NGRIDS-1)))

C---------READ GRID INFORMATION FOR A CHILD GRID
      DO LG=1,NGRIDS-1
          WRITE(IOUT,380) LG,IUDISCHILD(LG)
  380     FORMAT(1X,/1X,'CHILD GRID',I2,' DISCRETIZATION INPUT DATA',
     1    ' READ FROM UNIT',I3)
 
          CALL RDDIS(IUDISCHILD(LG),NCOLC(LG),NROWC(LG),NLAYC(LG),
     1     NPERC(LG),DELRC(LG,:),DELCC(LG,:),TOPC(LG,:,:),
     2     BOTMC(LG,:,:,:),PERLENC(LG,:),NSTPC(LG,:),TSMULTC(LG,:),
     3     ISSFLGC(LG,:),IUNITLGR(16))
           
          CLOSE (IUDISCHILD(LG))
      END DO      
      
C----------------------------------------------------------------------

C------READ MODPATH-LGR CONTROL FILE FOR GLOBAL OUTPUT FILES
      FNGLOBPL= ' '
      FNGLOBEP= ' '
      FNGLOBTS= ' '
      FNGLOBPROJ= ' '
      
      CALL GETMP(IUNITLGR(1),TEXT1,TEXT2,
     1 TEXT3,FNGLOBPL,FNGLOBEP,FNGLOBTS,FNGLOBPROJ,IUNITLGR(20),
     2 IUNITLGR(18),IUNITLGR(19),IUNITLGR(21))
      CLOSE(IUNITLGR(1))
      
      IF(ICMPCTP.EQ.2) THEN
         FMT = 'BINARY'
      ELSE 
         FMT = 'FORMATTED'
      END IF

C     OPEN FILES FOR SAVING GLOBAL OUTPUT      
      IF(FNGLOBPL.NE.' ') OPEN(IUNITLGR(20),FILE=FNGLOBPL,
     1 ACCESS='SEQUENTIAL',FORM=FMT,STATUS='REPLACE')
      IF(FNGLOBEP.NE.' ') OPEN(IUNITLGR(18),FILE=FNGLOBEP,
     1 ACCESS='SEQUENTIAL',FORM=FMT,STATUS='REPLACE')
      IF(FNGLOBTS.NE.' ') OPEN(IUNITLGR(19),FILE=FNGLOBTS,
     1 ACCESS='SEQUENTIAL',FORM=FMT,STATUS='REPLACE')
      IF(FNGLOBPROJ.NE.' ') OPEN(IUNITLGR(21),FILE=FNGLOBPROJ,
     1 ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='REPLACE')

C*******************BEGIN MODPATH-LGR SEQUENCES******************

C     INITIALIZE COUNTERS FOR SEQUENCES
      NUMITER=0
      
C     SET TALLY FOR PARTICLES MOVED TO ZERO
      PPARTNUM(:,:)=0
      CPARTNUM(:,:)=0
      
C     INITIALIZE EXIT STATUS
      IEXIT = 0      

C-------LOOP WITHIN EACH TIME STEP UNTIL PARTICLES DISCHARGE

      DO ! LOOP UNTIL NO PARTICLES ARE PASSED BACK TO THE PARENT GRID

C     INCREASE THE ITERATION COUNT      
      NUMITER=NUMITER+1
      
C     INITIALIZE COUNTERS FOR NUMBER OF PARTICLES ANY VALUE ABOVE 0
      NUMPARENTPART(:)=0
      NUMCHILDPART(:)=0

C     RESET THE NUMBER OF PASSED PARTICLES TO ZERO AT THE BEGINNING OF THE SEQUENCE      
      NUMCHILDPARTTOTAL(NUMITER)=0
      
      WRITE(IOUT,415)
      WRITE(IOUT,410)
      WRITE(IOUT,399) NUMITER
      
      WRITE(*,415)
      WRITE(*,410)       
      WRITE(*,399) NUMITER
      
  399 FORMAT('MODPATH-LGR SEQUENCE NUMBER =',I3)
  410 FORMAT('')    
  415 FORMAT(52('-'))
  420 FORMAT(32('-'))
      
C-----SET UP NAMES OF PARENT SCRATCH FILES
C     FILE NAME IS "PARENT_START_##" WHERE:
C     ## IS THE MODPATH-LGR SEQUENCE NUMBER 

C     CONVERT INTEGER NUMITER TO A CHARACTER
      WRITE(NITER,'(I4)') NUMITER
      NITER=ADJUSTL(NITER)
           
C     ASSIGN SCRATCH STARTING LOCATIONS, PATHLINE,ENDPOINT, AND TIME-SERIES FILE NAMES
C     FOR PARENT GRID    
      FNSTARTPARENTOLD='PARENT_START_'//TRIM(NITER)//'.DAT'
      FNPATHPARENTOLD='PARENT_PATHLINE_'//TRIM(NITER)//'.DAT'
      FNTSPARENTOLD='PARENT_TIMESERIES_'//TRIM(NITER)//'.DAT'
      FNENDPARENTOLD='PARENT_ENDPOINT_'//TRIM(NITER)//'.DAT'
      FNPROJPARENTOLD='PARENT_ADVOBS_'//TRIM(NITER)//'.DAT'

C     SAVE PARENT START SPECIFIED IN NAME FILE TO AN OUTPUT FILE FOR EACH MODPATH-LGR SEQUENCE
      OPEN(IUNITLGR(9),FILE=FNSTARTPARENTOLD,FORM='FORMATTED',
     1 STATUS='UNKNOWN')
      IF(NUMITER.EQ.1) THEN
           OPEN(IUNITLGR(10),FILE=FNSTARTPARENT,FORM='FORMATTED',
     1      STATUS='OLD')
           CALL WRITSCRATCH(IUNITLGR(10),IUNITLGR(9))
           CLOSE (IUNITLGR(10))
      ELSE
          CALL WRITEFSTART(IUNITLGR(10),IUNITLGR(9))
      END IF
      CLOSE (IUNITLGR(9))
    
C************************************************************************
C-----RUN MODPATH FOR THE PARENT GRID

  435 FORMAT('--------------------------------------')
  440 FORMAT('TRACKING PARTICLES FOR THE PARENT GRID')
!      WRITE(IUNITLGR(16),410)
!      WRITE(IUNITLGR(16),435)
!      WRITE(IUNITLGR(16),440)

      WRITE(*,435)
      WRITE(*,440)

      IDMODEL=1 ! for parent model
C     OPEN BLANK PARENT ENDPOINT FILE      
      OPEN(UNIT=IUNITLGR(17),STATUS='SCRATCH',
     1 ACCESS='SEQUENTIAL',FORM='UNFORMATTED') ! scratch endpoint file
      
      CALL MPATH(ISLGR,FNRSPPARENT,INTFLG,IUNITLGR(10),IUNITLGR(17),
     1 IUNITLGR(18),IUNITLGR(19),IUNITLGR(20),IUNITLGR(21),0,0,0,0,0,0,
     2 0,0,0,0,0,0,IBFLG,NGRIDS,NMPSTOP,IDMODEL,NUMITER)

C************************************************************************
C     DELETE PREVIOUS SCRATCH PARENT STARTING LOCATIONS     
      CLOSE(UNIT=IUNITLGR(10)) 
C     SAVE PARENT ENDPOINT, PATHLINE, TIME SERIES, AND PROJECTION TO AN OUTPUT FILE
C     FOR EACH MODPATH-LGR SEQUENCE

	INQUIRE(FILE=FNENDPARENT,EXIST=EXISTS)
	IF(EXISTS)THEN
          OPEN(IUNITLGR(8),FILE=FNENDPARENT,STATUS='OLD') ! parent endpoint from this run
          OPEN(IUNITLGR(9),FILE=FNENDPARENTOLD,STATUS='UNKNOWN') 
          CALL WRITSCRATCH(IUNITLGR(8),IUNITLGR(9))       ! write parent endpoint from this sequence
      CLOSE (IUNITLGR(8))
      CLOSE (IUNITLGR(9))
      ENDIF

      INQUIRE(FILE=FNTSPARENT,EXIST=EXISTS)
      IF(EXISTS)THEN
          OPEN(IUNITLGR(8),FILE=FNTSPARENT,STATUS='OLD') ! parent time-series from this run
          OPEN(IUNITLGR(9),FILE=FNTSPARENTOLD,STATUS='UNKNOWN')
          CALL WRITSCRATCH(IUNITLGR(8),IUNITLGR(9))      ! write parent time-series from this sequence
          CLOSE (IUNITLGR(8))
          CLOSE (IUNITLGR(9))
      ENDIF
	INQUIRE(FILE=FNPATHPARENT,EXIST=EXISTS)
	IF(EXISTS)THEN
          OPEN(IUNITLGR(8),FILE=FNPATHPARENT,STATUS='OLD') ! parent pathline from this run
          OPEN(IUNITLGR(9),FILE=FNPATHPARENTOLD,STATUS='UNKNOWN')
          CALL WRITSCRATCH(IUNITLGR(8),IUNITLGR(9))        ! write parent pathline from this sequence
          CLOSE (IUNITLGR(8))
          CLOSE (IUNITLGR(9))
      END IF
	INQUIRE(FILE=FNPROJPARENT,EXIST=EXISTS)
	IF(EXISTS)THEN
          OPEN(IUNITLGR(8),FILE=FNPROJPARENT,STATUS='OLD') ! parent projection from this run
          OPEN(IUNITLGR(9),FILE=FNPROJPARENTOLD,STATUS='UNKNOWN')
          CALL WRITSCRATCH(IUNITLGR(8),IUNITLGR(9))        ! write parent projection from this sequence
          CLOSE (IUNITLGR(8))
          CLOSE (IUNITLGR(9))
      END IF

C************************************************************************
C-----PROCESS PARENT GRID OUTPUT
C     READ PARENT ENDPOINT FILE, TRANSLATE COORDINATES, REWRITE TO CHILD START FILE

C     SET UP COUNTER FOR NUMBER OF PARTICLES
      NUMPARENTPARTTOTAL(NUMITER)=0

C     OPEN THE CHILD STARTING LOCATIONS FILE SPECIFIED IN NAME FILE IF THE FIRST SEQUENCE 
      DO LG=1,NGRIDS-1
          NCUNIT=LG+10
          IDMODEL_START=LG+1
          REWIND(IUNITLGR(17)) ! scratch parent endpoint file
C---------DETERMINE OFFSET BETWEEN PARENT AND CHILD GRIDS
          CALL GRIDOFFSET(NROWP,NCOLP,DELRP,DELCP,NPCBEG(LG),NPREND(LG),
     1    NCPP(LG),PXOFF(LG),PYOFF(LG),CXOFF(LG),CYOFF(LG))

C---------CALCULATE MAX X and Y COORDINATES FOR EACH CHILD CELL
          CALL GRIDCOORD(NCOLC(LG),NROWC(LG),DELRC(LG,:),DELCC(LG,:),
     1     CXMAX(LG,:),CYMAX(LG,:))

C-----------------------------------------------------------------------     
C----------READ CHILD MAIN DATA FILE TO READ OPTIONS

          CALL GETMAIN(IUNITLGR(7),FNMAINCHILD(LG),ICMPCTC)

C----------READ SCRATCH PARENT ENDPOINT FILE AND WRITE TO SCRATCH CHILD STARTING LOCATIONS FILE
C1A        IF THE PARENT ENDPOINT FILE HAS NO PARTICLES GOING TO THE CHILD GRID,
C1B        EG, NUMCHILDPART = 0, THEN PARTICLES ARE NOT ADDED TO THE CHILD STARTING LOCATIONS FILE.
          IF (NMPSTOP.NE.1) THEN
          
C            OPEN CHILD SCRATCH CHILD STARTING LOCATIONS FILE          
             OPEN(UNIT=IUNITLGR(NCUNIT),STATUS='SCRATCH',
     1        ACCESS='SEQUENTIAL',FORM='UNFORMATTED') 

             CALL EPFILE(IUNITLGR(17),IUNITLGR(NCUNIT),NCOLC(LG),
     1       NROWC(LG),NLAYC(LG),DELRC(LG,:),DELCC(LG,:),TOPC(LG,:,:),
     2       BOTMC(LG,:,:,:),CXOFF(LG),CYOFF(LG),IZONE,IBFLG(LG),
     3       NUMCHILDPART(LG),CXMAX(LG,:),CYMAX(LG,:))
  
          ELSE 
             NUMCHILDPART(LG)=0
          END IF

C----------IF FIRST ITERATION, APPEND THE STARTING LOCATIONS SPECIFIED IN THE CHILD NAME FILE
C1A          TO THE SCRATCH CHILD STARTING LOCATIONS FILE, WHICH CONTAINS STARTING LOCATIONS
C2A          CONTRIBUTED BY THE PARENT MODEL

          IF(NUMITER.EQ.1) THEN
             OPEN(IUNITLGR(10),FILE=FNSTARTCHILD(LG),STATUS='OLD', ! Child starting locations specified in name file
     1        ACCESS='SEQUENTIAL',FORM='FORMATTED')
             CALL WRITEUNFSTART(IUNITLGR(10),IUNITLGR(NCUNIT),
     1        IDMODEL_START)
             CLOSE (IUNITLGR(10))
          ENDIF

C-------KEEP A TALLY OF THE NUMBER OF PARTICLES PASSED TO CHILD GRIDS
C       AFTER EACH ITERATION
          CPARTNUM(NUMITER,LG)= NUMCHILDPART(LG)
          NUMCHILDPARTTOTAL(NUMITER)=NUMCHILDPARTTOTAL(NUMITER)+
     1     NUMCHILDPART(LG)

      END DO
C-----END WRITING NEW STARTING LOCATIONS FOR CHILD GRIDS
C     CLOSE THE SCRATCH ENDPOINT FILE               
      CLOSE(IUNITLGR(17))
      
C-------IF NOT THE FIRST ITERATION, AND NO PARTICLES MOVED FROM THE PARENT
C       TO CHILD GRIDS, THEN EXIT ITERATIONS
      IF(NUMCHILDPARTTOTAL(NUMITER) .EQ. 0 .AND. NUMITER .GT. 1) THEN
         WRITE(*,*) 'EXITING MODPATH RUNS BECAUSE NO PARTICLES ARE
     1 PASSED TO THE CHILD GRID(S)'
         IEXIT=1
C        PARTICLES CANNOT BE PASSED TO THE PARENT IN AN ITERATION IF THE CHILD DOESN'T RUN      
         GOTO 500 ! EXIT DO LOOP FOR MODPATH SEQUENCES
      ENDIF
      
C**********************************************************************
C     FOR CHILD GRIDS 
C************************************************************************
C     OPEN PARENT STARTING LOCATIONS FILE FOR NEXT SEQUENCE, WRITE CHILD GRID OUTPUT.
C     KEEP THIS FILE OPEN WHILE LOOPING THROUGH ALL CHILD GRIDS, WRITE NEW PARENT STARTING
C     LOCATIONS FROM EACH CHILD
      OPEN(UNIT=IUNITLGR(10),STATUS='SCRATCH',ACCESS='SEQUENTIAL',
     1     FORM='UNFORMATTED') 
 
      DO LG=1,NGRIDS-1
          NCUNIT=LG+10
C-----SET UP NAMES OF CHILD SCRATCH FILES
C     FILE NAME IS "CHILD_##_START_!!" WHERE:

C     ## IS THE CHILD GRID NUMBER
C     !! IS THE MODPATH-LGR SEQUENCE NUMBER 

C     CONVERT INTEGER NUMITER TO A CHARACTER
      WRITE(NLG,'(I4)') LG
      WRITE(NITER,'(I4)') NUMITER

      NLG=ADJUSTL(NLG)
      NITER=ADJUSTL(NITER)

      FNSTARTCHILDOLD(LG)='CHILD_'//TRIM(NLG)//'_START_'//TRIM(NITER)//
     1 '.DAT'
      FNPATHCHILDOLD(LG)='CHILD_'//TRIM(NLG)//'_PATHLINE_'//
     1 TRIM(NITER)//'.DAT'
      FNTSCHILDOLD(LG)='CHILD_'//TRIM(NLG)//'_TIMESERIES_'//
     1 TRIM(NITER)//'.DAT'      
      FNENDCHILDOLD(LG)='CHILD_'//TRIM(NLG)//'_ENDPOINT_'//TRIM(NITER)//
     1 '.DAT'
      FNPROJCHILDOLD(LG)='CHILD_'//TRIM(NLG)//'_ADVOBS_'//TRIM(NITER)//
     1 '.DAT'
C************************************************************************
C-----WRITE CHILD STARTING LOCATIONS TO A SCRATCH FILE
C     FILE NAME IS CHILDSTART## WHERE ## IS THE ITERATION NUMBER      
C     THE FIRST RUN IS ITERATION 1

      REWIND(IUNITLGR(NCUNIT))
          OPEN(IUNITLGR(9),FILE=FNSTARTCHILDOLD(LG),FORM='FORMATTED',
     1     STATUS='REPLACE')
          CALL WRITEFSTART(IUNITLGR(NCUNIT),IUNITLGR(9))
          CLOSE (IUNITLGR(9))  ! Child scratch starting locations
      REWIND(IUNITLGR(NCUNIT))          
C************************************************************************
C-----RUN MODPATH FOR THE CHILD GRID
  460 FORMAT('')
  470 FORMAT('-------------------------------------')
  480 FORMAT('TRACKING PARTICLES FOR THE CHILD GRID')
  490 FORMAT('TRACKING PARTICLES FOR CHILD GRID',I2) 

      WRITE(*,460)
      WRITE(*,470)
!      WRITE(IUNITLGR(16),460)
!      WRITE(IUNITLGR(16),470)
!      IF (ISLGR.EQ.0) THEN
!          WRITE(*,480)
!          WRITE(IUNITLGR(16),480)
!      ELSE
!          WRITE(*,490) LG  
!          WRITE(IUNITLGR(16),490) LG
!      END IF
      OPEN(UNIT=IUNITLGR(17),STATUS='SCRATCH',
     1 ACCESS='SEQUENTIAL',FORM='UNFORMATTED')      
      NCUNIT=LG+10
      IDMODEL=LG+1
      IF(NUMITER.EQ.1) IDMODEL_START=IDMODEL
      CALL MPATH(ISLGR,FNRSPCHILD(LG),INTFLG,IUNITLGR(NCUNIT),
     1 IUNITLGR(17),IUNITLGR(18),IUNITLGR(19),IUNITLGR(20),IUNITLGR(21),
     2 CXOFF(LG),CYOFF(LG),0,NROWP,NCOLP,NLAYP,DELRP,DELCP,TOPP,BOTMP,
     3 PXMAX,PYMAX,IBFLG,NGRIDS,NMPSTOP,IDMODEL,NUMITER)
    
C*********************************************************************
C     DELETE PREVIOUS SCRATCH CHILD STARTING LOCATIONS CONTRIBUTED BY PARENT MODEL 
      CLOSE(UNIT=IUNITLGR(NCUNIT))
      REWIND(IUNITLGR(17)) ! scratch child endpoint file
C     READ CHILD ENDPOINT FILE AND WRITE NEW STARTING LOCATIONS TO PARENT START FILE
C1    KEEP PARENT STARTING LOCATIONS FILE OPEN UNTIL ALL CHILD MODELS ARE PROCESSED.
      IF (NMPSTOP.NE.1) THEN
         CALL EPFILE(IUNITLGR(17),IUNITLGR(10),NCOLP,NROWP,NLAYP,
     1   DELRP,DELCP,TOPP,BOTMP,PXOFF(LG),PYOFF(LG),IZONE,IBFLG(LG),
     2   NUMPARENTPART(LG),PXMAX,PYMAX)
  495 CONTINUE    
      ELSE 
         NUMPARENTPART(LG)=0
      END IF
      CLOSE(IUNITLGR(17)) ! scratch endpoint file
      
C*********************************************************************
C     SAVE CHILD ENDPOINT, PATHLINE AND TIME-SERIES FILES TO OUTPUT FOR
C     EACH MODPATH-LGR SEQUENCE
	INQUIRE(FILE=FNENDCHILD(LG),EXIST=EXISTS)
	IF(EXISTS)THEN
          OPEN(IUNITLGR(8),FILE=FNENDCHILD(LG),STATUS='OLD')
          OPEN(IUNITLGR(9),FILE=FNENDCHILDOLD(LG),STATUS='UNKNOWN')
          CALL WRITSCRATCH(IUNITLGR(8),IUNITLGR(9))
          CLOSE (IUNITLGR(8))
          CLOSE (IUNITLGR(9))
      END IF

      INQUIRE(FILE=FNTSCHILD,EXIST=EXISTS)
      IF(EXISTS)THEN
          OPEN(IUNITLGR(8),FILE=FNTSCHILD(LG),STATUS='OLD')
          OPEN(IUNITLGR(9),FILE=FNTSCHILDOLD(LG),STATUS='UNKNOWN')
          CALL WRITSCRATCH(IUNITLGR(8),IUNITLGR(9))
          CLOSE (IUNITLGR(8))
          CLOSE (IUNITLGR(9))
      ENDIF

	INQUIRE(FILE=FNPATHCHILD,EXIST=EXISTS)
	IF(EXISTS)THEN
          OPEN(IUNITLGR(8),FILE=FNPATHCHILD(LG),STATUS='OLD')
          OPEN(IUNITLGR(9),FILE=FNPATHCHILDOLD(LG),STATUS='UNKNOWN')
          CALL WRITSCRATCH(IUNITLGR(8),IUNITLGR(9))
          CLOSE (IUNITLGR(8))
          CLOSE (IUNITLGR(9))
      END IF

	INQUIRE(FILE=FNPROJCHILD,EXIST=EXISTS)
	IF(EXISTS)THEN
          OPEN(IUNITLGR(8),FILE=FNPROJCHILD(LG),STATUS='OLD')
          OPEN(IUNITLGR(9),FILE=FNPROJCHILDOLD(LG),STATUS='UNKNOWN')
          CALL WRITSCRATCH(IUNITLGR(8),IUNITLGR(9))
          CLOSE (IUNITLGR(8))
          CLOSE (IUNITLGR(9))
      END IF

C-------KEEP A TALLY OF THE NUMBER OF PARTICLES PASSED TO PARENT GRID
C     FROM CHILD GRID LG AFTER EACH MODPATH-LGR SEQUENCE NUMITER

          PPARTNUM(NUMITER,LG)= NUMPARENTPART(LG)
          NUMPARENTPARTTOTAL(NUMITER)=NUMPARENTPARTTOTAL(NUMITER)+
     1     NUMPARENTPART(LG)
     
C-------IF NO PARTICLES MOVED FROM THE CHILD GRID(S) TO THE PARENT MODEL,
C       THEN EXIT ITERATIONS

C     END LOOP FOR RUNNING ONLY CHILD MODELS AND WRITING PARENT STARTING LOCATIONS FILE FROM CHILD GRID OUTPUT
      END DO
C************************************************************************
      
C     REWIND THE PARENT STARTING LOCATIONS FILE
      REWIND(IUNITLGR(10)) 
      
      IF(NUMPARENTPARTTOTAL(NUMITER) .EQ. 0) THEN 
         WRITE(*,*) 'EXITING MODPATH-LGR RUNS BECAUSE NO PARTICLES ARE 
     1 PASSED TO THE PARENT GRID'
         IEXIT=1
C        PARTICLES CANNOT BE PASSED TO THE PARENT IN AN ITERATION IF THE CHILD DOESN'T RUN      
         GOTO 500 ! EXIT DO LOOP FOR MODPATH-LGR SEQUENCES
      ENDIF

C************************************************************************
  500 CONTINUE
C     ECHO FINAL TALLY OF PARTICLES AFTER A MODPATH-LGR SEQUENCE

!  510 FORMAT('')       
!  515 FORMAT('----------------------------------------------------------
!     1--' /) 
  520 FORMAT('SUMMARY OF MODPATH-LGR SEQUENCE NUMBER =', I3)
  525 FORMAT('NUMBER OF PARTICLES PASSED FROM PARENT TO CHILD',I3,
     1 ' =', I8) 
  530 FORMAT('NUMBER OF PARTICLES PASSED FROM CHILD',I3,' TO PARENT =',
     1  I8) 

      WRITE(IUNITLGR(16),420)
      WRITE(IUNITLGR(16),410)      
      WRITE(IUNITLGR(16),520) NUMITER
      DO LG=1,NGRIDS-1
          WRITE(IUNITLGR(16),525) LG,NUMCHILDPART(LG)
          WRITE(IUNITLGR(16),530) LG,NUMPARENTPART(LG)
      END DO
      WRITE(IUNITLGR(16),410)           
     
      WRITE(*,410)       
      WRITE(*,415) 
      WRITE(*,410) 
      WRITE(*,520) NUMITER
      DO LG=1,NGRIDS-1
          WRITE(*,525) LG, NUMCHILDPART(LG)
          WRITE(*,530) LG, NUMPARENTPART(LG)
      END DO
      WRITE(*,*) 
      IF(IEXIT.EQ.1) EXIT      
C***********************************************************************
C-------END ITERATIONS HERE

C     STOP SEQUENCES IF MORE THAN MAXSEQ NUMBER OF SEQUENCES
      IF(NUMITER.EQ.MAXSEQ) THEN
         WRITE(*,397) MAXSEQ
         WRITE(*,*) ' ' 
  397    FORMAT('MAXIMUM NUMBER OF',1X,I3,1X,'SEQUENCES EXCEEDED') 
         EXIT
      END IF

      END DO ! END LOOP THROUGH MODPATH-LGR SEQUENCES
      
      CLOSE(IUNITLGR(10)) ! close starting locations file
 
C***********************************************************************
     
      
C=====SORT GLOBAL OUTPUT FILES===================

C------------------------------------------------      
C     FINISH GLOBAL ENDPOINT FILE
C     REMOVE DUPLICATE ENDPOINTS FOR EACH PARTICLE FROM LEAVING MODELS    
C     GET HEADER FROM ENDPOINT FILE FOR GLOBAL FILE     
      CLOSE (IUNITLGR(18))

      IF(FNGLOBEP.NE.' ') THEN      
C        GET HEADER FROM ENDPOINT FILE FOR GLOBAL FILE 
	   INQUIRE(FILE=FNENDPARENT,EXIST=EXISTS)
	   IF(EXISTS)THEN
	      IF(ICMPCTP.EQ.2) THEN
               OPEN(IUNITLGR(8),FILE=FNENDPARENT,FORM=FMT,
     1          STATUS='OLD')
               READ(IUNITLGR(8)) HEADERLINE,TREF	       
	      ELSE
               OPEN(IUNITLGR(8),FILE=FNENDPARENT,STATUS='UNKNOWN')
               READ(IUNITLGR(8),'(A)') EPHEADER
            END IF
            CLOSE (IUNITLGR(8))
         END IF
         
         OPEN(IUNITLGR(18),FILE=FNGLOBEP,FORM=FMT)
         OPEN(IUNITLGR(9),FORM=FMT,STATUS='SCRATCH')      
         CALL SORTEP(IUNITLGR(18),IUNITLGR(9),ICMPCTP)
         CLOSE(IUNITLGR(18))
         OPEN(IUNITLGR(18),FILE=FNGLOBEP,FORM=FMT,STATUS='OLD')
         REWIND(IUNITLGR(9))
         
         IF(ICMPCTP.EQ.2) THEN
            WRITE(IUNITLGR(18)) HEADERLINE,TREF
            CALL WRITEUNFEP(IUNITLGR(9),IUNITLGR(18))
         ELSE
            WRITE(IUNITLGR(18),'(T1,A)') EPHEADER
            CALL WRITSCRATCH(IUNITLGR(9),IUNITLGR(18))
         END IF
 
         CLOSE (IUNITLGR(18)) ! ENDPOINT FILE
         CLOSE (IUNITLGR(9)) ! SCRATCH FILE 
      END IF      
C---------------------------------------------------------------
C     FINISH GLOBAL TIME-SERIES FILE 
C     LEAVE OUT UNLESS IT IS REQUESTED
      CLOSE(IUNITLGR(19))

      IF(FNGLOBTS.NE.' ') THEN
C        GET HEADER FROM TIME-SERIES FILE FOR GLOBAL FILE     
	   INQUIRE(FILE=FNTSPARENT,EXIST=EXISTS)
	   IF(EXISTS)THEN
	      IF(ICMPCTP.EQ.2) THEN
               OPEN(IUNITLGR(8),FILE=FNTSPARENT,FORM=FMT,
     1          STATUS='OLD')
               READ(IUNITLGR(8)) HEADERLINE,TREF
	      ELSE
               OPEN(IUNITLGR(8),FILE=FNTSPARENT,STATUS='UNKNOWN')
               READ(IUNITLGR(8),'(A)') TSHEADER
            END IF	   
            CLOSE (IUNITLGR(8))
         END IF
         OPEN(IUNITLGR(19),FILE=FNGLOBTS,FORM=FMT)
         OPEN(IUNITLGR(9),FORM=FMT,STATUS='SCRATCH')
         IF(ICMPCTP.EQ.2) THEN
            CALL WRITEUNFTS(IUNITLGR(19),IUNITLGR(9))
         ELSE IF(ICMPCTP.EQ.0) THEN
            CALL WRITSCRATCH(IUNITLGR(19),IUNITLGR(9))
         END IF
                  
!         CALL SORTTS(IUNITLGR(19),IUNITLGR(9),ICMPCTP)
         CLOSE(IUNITLGR(19))
         OPEN(IUNITLGR(19),FILE=FNGLOBTS,FORM=FMT,STATUS='OLD')
         REWIND(IUNITLGR(9))
         
         IF(ICMPCTP.EQ.2) THEN
            WRITE(IUNITLGR(19)) HEADERLINE,TREF
            CALL WRITEUNFTS(IUNITLGR(9),IUNITLGR(19))
         ELSE
            WRITE(IUNITLGR(19),'(T1,A)') TSHEADER
            CALL WRITSCRATCH(IUNITLGR(9),IUNITLGR(19))
         END IF         
         CLOSE (IUNITLGR(19)) ! PATHLINE FILE 
         CLOSE (IUNITLGR(9)) ! SCRATCH FILE
      END IF

C--------------------------------------------------------------- 
C     FINISH GLOBAL PATHLINE FILE
C     SORT EACH FILE BY STRESS PERIOD AND TIME STEP

      CLOSE(IUNITLGR(20))
      IF(FNGLOBPL.NE.' ') THEN
C        GET HEADER FROM PATHLINE FILE FOR GLOBAL FILE     
	   INQUIRE(FILE=FNPATHPARENT,EXIST=EXISTS)
	   IF(EXISTS)THEN
	      IF(ICMPCTP.EQ.2) THEN
               OPEN(IUNITLGR(8),FILE=FNPATHPARENT,FORM=FMT,
     1          STATUS='OLD')
               READ(IUNITLGR(8)) HEADERLINE,TREF	       
	      ELSE
               OPEN(IUNITLGR(8),FILE=FNPATHPARENT,STATUS='UNKNOWN')
               READ(IUNITLGR(8),'(A)') PLHEADER
            END IF	   
            CLOSE (IUNITLGR(8))
         END IF
         
         OPEN(IUNITLGR(20),FILE=FNGLOBPL,FORM=FMT)
         OPEN(IUNITLGR(9),FORM=FMT,STATUS='SCRATCH')      
         CALL SORTPL(IUNITLGR(20),IUNITLGR(9),ICMPCTP,IUNITLGR(10),
     1    NGRIDS,FNSTARTPARENT,FNSTARTCHILD)
         CLOSE(IUNITLGR(20))
         OPEN(IUNITLGR(20),FILE=FNGLOBPL,FORM=FMT,STATUS='OLD')
         REWIND(IUNITLGR(9))

         IF(ICMPCTP.EQ.2) THEN
            WRITE(IUNITLGR(20)) HEADERLINE,TREF
            CALL WRITEUNFEP(IUNITLGR(9),IUNITLGR(20))
         ELSE
            WRITE(IUNITLGR(20),'(T1,A)') PLHEADER
            CALL WRITSCRATCH(IUNITLGR(9),IUNITLGR(20))
         END IF

         CLOSE (IUNITLGR(20)) ! PATHLINE FILE 
         CLOSE (IUNITLGR(9)) ! SCRATCH FILE
      END IF
C--------------------------------------------------------------- 
C     FINISH GLOBAL PROJECTIONS FILE      
C     LEAVE OUT UNLESS IT IS REQUESTED
      CLOSE(IUNITLGR(21))
!
!      IF(FNGLOBPROJ.NE.' ') THEN
!	INQUIRE(FILE=FNGLOBTS,EXIST=EXISTS)
!	IF(EXISTS)THEN     
!C        GET HEADER FROM TIME-SERIES FILE FOR GLOBAL FILE     
!	   INQUIRE(FILE=FNTSPARENT,EXIST=EXISTS)
!	   IF(EXISTS)THEN
!            OPEN(IUNITLGR(8),FILE=FNTSPARENT,STATUS='UNKNOWN')      
!            READ(IUNITLGR(8),'(A)') TSHEADER
!            CLOSE (IUNITLGR(8))
!         END IF
!         OPEN(IUNITLGR(21),FILE=FNGLOBPROJ)
!         OPEN(IUNITLGR(9),STATUS='SCRATCH')      
!         CALL SORTADVOBS(IUNITLGR(21),IUNITLGR(9))
!         CLOSE(IUNITLGR(21))
!         OPEN(IUNITLGR(21),FILE=FNGLOBPROJ,STATUS='OLD')
!         REWIND(IUNITLGR(9))
!         WRITE(IUNITLGR(21),'(T1,A)') TSHEADER
!         CALL WRITSCRATCH(IUNITLGR(9),IUNITLGR(19)) 
!         CLOSE (IUNITLGR(21)) ! PROJECTION FILE 
!         CLOSE (IUNITLGR(9)) ! SCRATCH FILE
!      END IF
!      CLOSE(IUNITLGR(20))
!      END IF
      
C     ECHO FINAL TALLY OF PARTICLES AFTER LAST ITERATION

      WRITE(IUNITLGR(16),415)
      WRITE(IUNITLGR(16),*) 'END MODPATH-LGR SIMULATION'
      WRITE(IUNITLGR(16),410) 

      WRITE(*,415)
      WRITE(*,*) 'END MODPATH-LGR SIMULATION'
      WRITE(*,410)
c      WRITE(*,520) NUMITER
c      DO LG=1,NGRIDS-1
c          WRITE(*,525) LG,NUMCHILDPART(LG)
c          WRITE(*,530) LG,NUMPARENTPART(LG)
c      END DO
c      WRITE(*,*)  


  950 FORMAT(T4,'SUMMARY OF PARTICLE TRACKING BETWEEN GRIDS'
     1 / 65('-'))
  960 FORMAT('MODPATH-LGR SEQUENCE NUMBER',T27,'PASSED TO CHILD GRIDS', 
     1 T50,'PASSED TO PARENT GRID'/23('-'),T27,21('-'),T50,21('-')/ )
  970 FORMAT(I3,T30,I3,T50,I3)
      WRITE(IUNITLGR(16),950)
      WRITE(IUNITLGR(16),960)

      DO ITER=1,NUMITER
      WRITE(IUNITLGR(16),970) ITER,NUMCHILDPARTTOTAL(ITER),
     1 NUMPARENTPARTTOTAL(ITER)    
      END DO
     
         
C     CLOSE FILE FOR INPUT ECHO      
      CLOSE (IUNITLGR(16))      

C***********************************************************************

C     END PROGRAM MPLGR
      END
      