      SUBROUTINE PCBC(FNBUDOLD,FNBUDNEW,FNLGR,IN,IOUT,INLGR)
C  TRANSFORM AND RE-WRITE PARENT BUDGET READ FROM MODFLOW UNFORMATTED 
C  FILES.  DOUBLE FLUXES PARALLEL TO THE GRID BOUNDARY.  
C  MODIFIED FROM PROGRAM PBUF ORIGINALLY WRITTEN BY:
C     WRITTEN BY ARLEN W. HARBAUGH
C     MODIFIED BY NED BANTA
C
C  INPUT COMES ON UNIT 10 -- IE THE MODEL DATA FILE, WHICH HOLDS
C  THE UNFORMATTED FLOW FILE
C  OUTPUT GOES TO UNIT 11.
C
      CHARACTER*40 VERSION
      PARAMETER (VERSION='PCBC2LGR version 1.0 2007/06/25')
      ALLOCATABLE BUF(:)
      ALLOCATABLE IBUF(:)
      ALLOCATABLE NPRBEG(:), NPREND(:), NPCBEG(:), NPCEND(:), NPLBEG(:),
     &            NPLEND(:) 
      CHARACTER*200 LINE
      CHARACTER*80 FNBUDOLD, FNBUDNEW, FNLGR, INNAMU, OUTNAMU
      CHARACTER*16 TEXT(6)
      LOGICAL ALLOC, FILEX
      DIMENSION QV(6)
      INCLUDE 'openspec.inc'
C     ------------------------------------------------------------------
  480 FORMAT(/,1X,A,/)
  490 FORMAT(' Enter the name of a file for writing the transformed ',
     &       'budget data')
  500 FORMAT(' ERROR: FILE DOES NOT EXIST: ',A)
  505 FORMAT(' MODEL CONTAINS TOO MANY COLUMNS FOR PBUF TO HANDLE BY',
     &' DEFAULT',/,' -- SPECIFY FORMATS AFTER OUTPUT FILE NAME')
  510 FORMAT(' KSTP,KPER,TEXT,NCOL,NROW,NLAY:',2(1X,I4),1X,A,3(1X,I5))
  515 FORMAT(' READING AND WRITING BUDGET COMPONENT: ',A)
  520 FORMAT(' NCOL = ',I10,' NROW = ',I10,' NLAY = ',I10,/,
     &' NOT ENOUGH MEMORY AVAILABLE -- STOP EXECUTION')
  530 FORMAT(' NCB,DELT,PERTIM,TOTIM: ',I3,1P,3(1X,E14.7))
  540 FORMAT(' ERROR: Input and output file names are the same')

      OPEN(UNIT=IN,FILE=FNBUDOLD,STATUS='OLD',FORM=FORM,ACCESS=ACCESS)

      OPEN(UNIT=IOUT,FILE=FNBUDNEW,FORM=FORM,ACCESS=ACCESS)

      OPEN(UNIT=INLGR,FILE=FNLGR,STATUS='OLD')
C
C     PARSE LGR FILE AND FIND THE NUMBER OF GRIDS
      CALL LGRPARSE1(INLGR,NGRIDS)
      ALLOCATE(NPRBEG(NGRIDS),NPREND(NGRIDS),NPCBEG(NGRIDS),
     1         NPCEND(NGRIDS),NPLBEG(NGRIDS),NPLEND(NGRIDS))
C     GET BEGINNING AND ENDING LAY, ROW, COL OF REFINEMENT FROM LGR FILE
      CALL LGRPARSE2(INLGR,NPRBEG,NPREND,NPCBEG,NPCEND,NPLBEG,NPLEND,
     1               NGRIDS)
      CLOSE(INLGR)
      ALLOC = .FALSE.
   50 CONTINUE
C
C     READ A HEADER RECORD FROM UNFORMATTED FILE
cswm      READ(IN,ERR=1000) KSTP,KPER,TEXT(1),NCOL,NROW,NLAY
      READ(IN,ERR=1000,END=1000) KSTP,KPER,TEXT(1),NCOL,NROW,NLAY
      WRITE(IOUT) KSTP,KPER,TEXT(1),NCOL,NROW,NLAY

C     ALLOCATE MEMORY AND OPEN OUTPUT FILE, IF NEEDED
      IF (.NOT.ALLOC) THEN
        NT=NCOL*NROW*ABS(NLAY)
        NRC=NCOL*NROW
        ALLOCATE (BUF(NT),STAT=JBUFSTAT)
        ALLOCATE (IBUF(NRC),STAT=IBUFSTAT)
        IF (JBUFSTAT.NE.0 .OR. IBUFSTAT.NE.0) THEN
          WRITE(*,520) NCOL,NROW,NLAY
          STOP
        ENDIF
        ALLOC = .TRUE.
      ENDIF
C  negative NLAY indicates COMPACT BUDGET
      IF (NLAY.LT.0) THEN
        NLAY=-NLAY
C  Read the extra record that defines the compact type
        READ(IN) NCB,DELT,PERTIM,TOTIM
        WRITE(IOUT) NCB,DELT,PERTIM,TOTIM
      ELSE
        NCB = 1
        DELT = -9999.9
        PERTIM = -9999.9
        TOTIM = -9999.9
      ENDIF
C
      IF (NCB.EQ.1) THEN
C  Full 3-D array  SWM SWM SWM LEFT OFF HERE!!!!!
        READ(IN) (BUF(I),I=1,NT)
        CALL ADJSTFLX(BUF,NCOL,NROW,NLAY,IOUT,TEXT(1),NPRBEG,NPCBEG,
     1                NPLBEG,NPREND,NPCEND,NPLEND,NGRIDS)
        WRITE(IOUT) (BUF(I),I=1,NT)

      ELSEIF (NCB.EQ.2) THEN
C  List without auxiliary values
        READ(IN) NLIST
        WRITE(IOUT) NLIST
        DO 100 L=1,NLIST
          READ(IN) ICRL,Q
          WRITE(IOUT) ICRL,Q
100     CONTINUE
C
      ELSEIF (NCB.EQ.3) THEN
C  2-D array plus a layer indicator array
        READ(IN) (IBUF(I),I=1,NRC)
        WRITE(IOUT) (IBUF(I),I=1,NRC)
        READ(IN) (BUF(I),I=1,NRC)
        WRITE(IOUT) (BUF(I),I=1,NRC)
C
      ELSEIF (NCB.EQ.4) THEN
C  2-D array for layer 1
        READ(IN) (BUF(I),I=1,NRC)
        WRITE(IOUT) (BUF(I),I=1,NRC)
C
      ELSEIF (NCB.EQ.5) THEN
C  List with auxiliary values
        READ(IN) NVAL
        WRITE(IOUT) NVAL
        IF (NVAL.GT.1) THEN
          READ(IN) (TEXT(N),N=2,NVAL)
          WRITE(IOUT) (TEXT(N),N=2,NVAL)
        ENDIF
        READ(IN) NLIST
        WRITE(IOUT) NLIST
        DO 200 L=1,NLIST
          READ(IN) ICRL,(QV(N),N=1,NVAL)
          WRITE(IOUT) ICRL,(QV(N),N=1,NVAL)
  200   CONTINUE
      END IF 
C
      GO TO 50
C
 1000 CONTINUE
      DEALLOCATE(BUF,IBUF)
      DEALLOCATE(NPRBEG,NPREND,NPCBEG,NPCEND,NPLBEG,NPLEND)
      CLOSE(IN)
      CLOSE(IOUT)
      
      RETURN
      END
C ======================================================================
      SUBROUTINE LGRPARSE1(INLGR,NGRIDS)
C     GET THE NUMBER OF GRIDS FROM THE LGR FILE
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*200 LINE
C     ------------------------------------------------------------------
      CALL URDCOM(INLGR,0,LINE)
      ICOL=1
      CALL URWORD(LINE,ICOL,ISTART,ISTOP,1,N,R,0,0) !LGR keyword
      CALL URDCOM(INLGR,0,LINE)
      ICOL=1
      CALL URWORD(LINE,ICOL,ISTART,ISTOP,2,NGRIDS,R,0,0) !NGRIDS
C
      RETURN
      END
C ======================================================================
C     GET BEGINNING AND ENDING LAY, ROW, COL OF REFINEMENT FROM LGR FILE
      SUBROUTINE LGRPARSE2(INLGR,NPRBEG,NPREND,NPCBEG,NPCEND,NPLBEG,
     &                     NPLEND,NGRIDS)
C     GET BEGINNING AND ENDING LAY, ROW, COL OF REFINEMENT FROM LGR FILE
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*200 LINE
      DIMENSION NPRBEG(NGRIDS),NPREND(NGRIDS),NPCBEG(NGRIDS)
      DIMENSION NPCEND(NGRIDS),NPLBEG(NGRIDS),NPLEND(NGRIDS)
      DIMENSION NCPP(NGRIDS),NCPPL(NGRIDS,99)
C     ------------------------------------------------------------------
C
C     PARSE OUT PARENT INFO
      CALL URDCOM(INLGR,0,LINE)
      ICOL=1
      CALL URWORD(LINE,ICOL,ISTART,ISTOP,0,N,R,0,0) !Parent Name file
      CALL URDCOM(INLGR,0,LINE)
      ICOL=1
      CALL URWORD(LINE,ICOL,ISTART,ISTOP,0,N,R,0,0) !GRIDSTATUS
      CALL URDCOM(INLGR,0,LINE)
      ICOL=1
      CALL URWORD(LINE,ICOL,ISTART,ISTOP,2,N,R,0,0) !IUPBHSV
C
      DO LG=1,NGRIDS-1
        CALL URDCOM(INLGR,0,LINE)
        ICOL=1
        CALL URWORD(LINE,ICOL,ISTART,ISTOP,0,N,R,0,0) !Child Name file
        CALL URDCOM(INLGR,0,LINE)
        ICOL=1
        CALL URWORD(LINE,ICOL,ISTART,ISTOP,0,N,R,0,0) !GRIDSTATUS
        CALL URDCOM(INLGR,0,LINE)
        ICOL=1
        CALL URWORD(LINE,ICOL,ISTART,ISTOP,2,N,R,0,0) !ISHFLG
        CALL URDCOM(INLGR,0,LINE)
        ICOL=1
        CALL URWORD(LINE,ICOL,ISTART,ISTOP,2,N,R,0,0) !MXLGRITER
        CALL URDCOM(INLGR,0,LINE)
        ICOL=1
        CALL URWORD(LINE,ICOL,ISTART,ISTOP,3,N,R,0,0) !RELAXH
        CALL URDCOM(INLGR,0,LINE)
        ICOL=1
        CALL URWORD(LINE,ICOL,ISTART,ISTOP,3,N,R,0,0) !HCLOSELGR
        CALL URDCOM(INLGR,0,LINE)
        READ (LINE,*)  NPLBEG(LG),NPRBEG(LG),NPCBEG(LG)
        CALL URDCOM(INLGR,0,LINE)
        READ (LINE,*)  NPLEND(LG),NPREND(LG),NPCEND(LG)
        CALL URDCOM(INLGR,0,LINE)
        READ (LINE,*)  NCPP(LG)
        CALL URDCOM(INLGR,0,LINE)
        READ (LINE,*) (NCPPL(LG,NPL), NPL=1,NPLEND(LG))


      ENDDO
C
      RETURN
      END
C ======================================================================
      SUBROUTINE ADJSTFLX(BUF,NCOL,NROW,NLAY,IOUT,TEXT,NPRB,NPCB,
     1                    NPLB,NPRE,NPCE,NPLE,NGRIDS)
C     ADJUST FLUXES ALONG THE GRID ITERFACE TO ACCOUNT FOR HALF CELLS
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*16 TEXT
      CHARACTER*200 LINE
      DIMENSION BUF(NCOL,NROW,NLAY)
      DIMENSION NPRB(NGRIDS),NPRE(NGRIDS),NPCB(NGRIDS)
      DIMENSION NPCE(NGRIDS),NPLB(NGRIDS),NPLE(NGRIDS)
C     ------------------------------------------------------------------
C1------PARSE TEXT TO FIND OUT WHAT FLUX IS STORED IN BUF
      ICOL=1
      CALL URWORD(TEXT,ICOL,ISTART,ISTOP,0,N,R,0,0) 
      CALL URWORD(TEXT,ICOL,ISTART,ISTOP,0,N,R,0,0) 
C
C2------LOOP THROUGH ALL GRIDS AND ALTER FLUXES. 
C2------CHECK TO SEE THIS GRID HAS A BOTTOM INTERFACE
      DO LG=1,NGRIDS-1
         
        IBOTFLG = 1
        NPRBEG=NPRB(LG)
        NPCBEG=NPCB(LG)
        NPLBEG=NPLB(LG)
        NPREND=NPRE(LG)
        NPCEND=NPCE(LG)
        NPLEND=NPLE(LG)
        IF(NPLBEG .EQ. NPLEND .OR.
     &     NPLEND .EQ. NLAY) THEN
          IBOTFLG=0
          LASTLAY=NPLEND
        ELSE
          LASTLAY=NPLEND-1
        ENDIF
C3A-----RIGHT OR FRONT FACE FLUX
        IF(TEXT(ISTART:ISTOP) .EQ. 'RIGHT' .OR. 
     &    TEXT(ISTART:ISTOP) .EQ. 'FRONT')THEN
          IF(TEXT(ISTART:ISTOP) .EQ. 'RIGHT')THEN
            DO K=1,LASTLAY
             DO J = NPCBEG+1,NPCEND-2 !should this be NPCBEG, NPCEND-1?
                BUF(J,NPRBEG,K) = 2.0*BUF(J,NPRBEG,K) 
                BUF(J,NPREND,K) = 2.0*BUF(J,NPREND,K) 
              ENDDO
            ENDDO
          ENDIF
          IF(TEXT(ISTART:ISTOP) .EQ. 'FRONT')THEN
            DO K=1,LASTLAY
              DO I = NPRBEG+1,NPREND-2
                BUF(NPCBEG,I,K) = 2.0*BUF(NPCBEG,I,K) 
                BUF(NPCEND,I,K) = 2.0*BUF(NPCEND,I,K) 
              ENDDO
            ENDDO
          ENDIF
C3B-----CHECK IF THERE IS A BOTTOM INTERFACE.
C3B-----DOUBLE FLUXES IN INTERIOR, 4/3 ON APPROPRIATE EDGE DIRECTIONS.  
C3B-----SKIP CORNERS
          IF(IBOTFLG .EQ. 1)THEN
            DO I = NPRBEG,NPREND
              DO J = NPCBEG,NPCEND
C SKIP CORNER
                IF((I .EQ. NPRBEG .OR. I .EQ. NPREND) .AND. 
     &             (J .EQ. NPCBEG .OR. J .EQ. NPCEND)) THEN
                  CYCLE 
C EDGE
                ELSEIF(I .EQ. NPRBEG .OR. I .EQ. NPREND) THEN 
                  IF(TEXT(ISTART:ISTOP) .EQ. 'RIGHT') THEN
                    IF(J .LE. NPCEND-2)
     &                 BUF(J,I,K) = 4.0/3.0*BUF(J,I,NPLEND) 
                  ELSEIF(I .NE. NPREND) THEN
                    BUF(J,I,K) = 2.0*BUF(J,I,NPLEND) 
                  ENDIF
                ELSEIF(J .EQ. NPCBEG .OR. J .EQ. NPCEND) THEN
                  IF(TEXT(ISTART:ISTOP) .EQ. 'FRONT') THEN 
                    IF(I .LE. NPREND-2)
     &                 BUF(J,I,K) = 4.0/3.0*BUF(J,I,NPLEND) 
                  ELSEIF(J .NE. NPCEND) THEN
                    BUF(J,I,K) = 2.0*BUF(J,I,NPLEND) 
                  ENDIF
C INTERIOR
                ELSE
                  BUF(J,I,K) = 2.0*BUF(J,I,NPLEND) 
                ENDIF
              ENDDO
            ENDDO
          ENDIF
C4A-----BOTTOM FACE FLUX
        ELSEIF(TEXT(ISTART:ISTOP) .EQ. 'LOWER')THEN 
          DO K=1,MAX(1,NPLEND-1)
            DO I = NPRBEG,NPREND
              DO J = NPCBEG,NPCEND
C CORNERS
                IF((I .EQ. NPRBEG .OR. I .EQ. NPREND) .AND. 
     &             (J .EQ. NPCBEG .OR. J .EQ. NPCEND)) THEN
                  BUF(J,I,K) = 4.0/3.0*BUF(J,I,K) 
C EDGE
                ELSEIF((I .EQ. NPRBEG .OR. I .EQ. NPREND) .OR. 
     &                 (J .EQ. NPCBEG .OR. J .EQ. NPCEND)) THEN
                  BUF(J,I,K) = 2.0*BUF(J,I,K) 
                ENDIF
              ENDDO
            ENDDO
          ENDDO
C
        ENDIF
C
      ENDDO
      RETURN
      END